package cn.iocoder.yudao.module.wealth.controller.admin.receipt;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.CurrencyRespDTO;
import cn.iocoder.yudao.module.ecw.api.currency.dto.ExchangeRateRespDTO;
import cn.iocoder.yudao.module.wealth.convert.receipt.ReceiptConvert;
import cn.iocoder.yudao.module.wealth.convert.receiptAccount.ReceiptAccountConvert;
import cn.iocoder.yudao.module.wealth.convert.receiptApproval.ReceiptApprovalConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receipt.ReceiptDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptAccount.ReceiptAccountDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptApproval.ReceiptApprovalDO;
import cn.iocoder.yudao.module.wealth.job.UpdateReceiptNewFieldTask;
import cn.iocoder.yudao.module.wealth.service.receipt.ReceiptService;
import cn.iocoder.yudao.module.wealth.service.receiptAccount.ReceiptAccountService;
import cn.iocoder.yudao.module.wealth.service.receiptApproval.ReceiptApprovalService;
import cn.iocoder.yudao.module.wealth.service.receivable.ReceivableService;
import cn.iocoder.yudao.module.wealth.vo.receipt.*;
import cn.iocoder.yudao.module.wealth.vo.receiptAccount.ReceiptAccountBackVO;
import cn.iocoder.yudao.module.wealth.vo.receiptApproval.ReceiptApprovalBackVO;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableBackVO;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableQueryVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFClientAnchor;
import org.apache.poi.xssf.usermodel.XSSFDrawing;
import org.apache.poi.xssf.usermodel.XSSFPicture;
import org.apache.poi.xssf.usermodel.XSSFShape;
import org.springframework.core.io.ClassPathResource;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.module.wealth.enums.ErrorCodeConstants.PAYMENT_NOT_EXISTS;

@Validated
@RestController
@Api(tags = "管理后台 - 收款单")
@RequestMapping("/ecw/receipt")
@Slf4j
public class ReceiptController {
    @Resource
    private ReceiptService receiptService;
    @Resource
    private ReceivableService receivableService;
    @Resource
    private ReceiptAccountService receiptAccountService;
    @Resource
    private CurrencyApi currencyApi;
    @Resource
    private ReceiptApprovalService receiptApprovalService;
    @Resource
    private UpdateReceiptNewFieldTask updateReceiptNewFieldTask;


    @PostMapping("/page")
    @ApiOperation("获得收款单分页")
    //@PreAuthorize("@ss.hasPermission('ecw:receipt:query')")
    public CommonResult<PageResult<ReceiptBackVO>> getReceiptPage(@RequestBody @Valid ReceiptQueryVO query, PageVO page) {
        //添加收款单明细状态计算功能
        //添加订单号和提单号的关联展示
        PageResult<ReceiptBackVO> pageResult = receiptService.getReceiptPage(query, page);
        return success(pageResult);
    }

    @GetMapping("/receivablePage")
    @ApiOperation("获取客户未收款列表")
    //@PreAuthorize("@ss.hasPermission('ecw:receipt:receivableQuery')")
    public CommonResult<PageResult<ReceivableBackVO>> getReceivablePage(@Valid ReceivableQueryVO query, PageVO page) {
        PageResult<ReceivableBackVO> pageResult = receivableService.receivablePage(query, page);
        return success(pageResult);
    }

    @PostMapping("/create")
    @ApiOperation("创建收款单")
    @Idempotent(timeout = 5)
    public CommonResult<Long> createReceipt(@Valid @RequestBody ReceiptCreateReqVO createReqVO) {
        return success(receiptService.createReceipt(createReqVO));
    }

    @PostMapping("/cancelFinanceReceiptApproval")
    @ApiOperation("取消收款单审核流程")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinanceReceiptApproval(@RequestBody FinanceReceiptCancelVo createReqVO) {
        receiptService.cancelFinanceReceiptApproval(createReqVO);
        return success(true);
    }

    @PostMapping("/financeApproveNo")
    @ApiOperation("收款单发起反审核")
    @Idempotent(timeout = 5)
    public CommonResult receiptFinanceApproveNo(@RequestBody FinanceReceiptVo financeReceiptVo) {
        receiptService.createFinanceApproveNo(financeReceiptVo);
        return success(true);
    }


    /*
     批量收款单反审核
     */
    @PostMapping("/financeApproveNo2")
    @ApiOperation("收款单发起反审核")
    @Idempotent(timeout = 5)
    public CommonResult receiptFinanceApproveNo2(
            @RequestBody List<FinanceReceiptVo> financeReceiptVoList) {

        if (CollectionUtil.isEmpty(financeReceiptVoList)) {
            return success(true);
        }

        for (FinanceReceiptVo financeReceiptVo : financeReceiptVoList
        ) {
            receiptService.createFinanceApproveNo(financeReceiptVo);
        }
        return success(true);
    }

    @PostMapping("/cancelFinanceReceiptApprovalNo")
    @ApiOperation("取消收款单反审核")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinanceReceiptApprovalNo(
            @RequestBody FinanceReceiptCancelVo createReqVO) {
        receiptService.cancelFinanceReceiptApprovalNo(createReqVO);
        return success(true);
    }


    @PostMapping("/financeReceiptWriteOff")
    @ApiOperation("收款单核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> createFinanceReceiptWriteOff(@RequestBody FinanceReceiptVo financeReceiptVo) {
        receiptService.createFinanceReceiptWriteOff(financeReceiptVo);
        return success(true);
    }

    /*
     批量收款单核销
     */
    @PostMapping("/financeReceiptWriteOff2")
    @ApiOperation("收款单核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> createFinanceReceiptWriteOff2(
            @RequestBody List<FinanceReceiptVo> financeReceiptVoList) {
        //判断List是否为空
        if (CollectionUtil.isEmpty(financeReceiptVoList)) {
            return success(true);
        }

        for (FinanceReceiptVo financeReceiptVo : financeReceiptVoList
        ) {
            receiptService.createFinanceReceiptWriteOff(financeReceiptVo);
        }
        return success(true);
    }

    @PostMapping("/cancelFinanceReceiptWriteOff")
    @ApiOperation("取消收款单核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinanceReceiptWriteOff(@RequestBody FinanceReceiptCancelVo createReqVO) {
        receiptService.cancelFinanceReceiptWriteOff(createReqVO);
        return success(true);
    }

    @PostMapping("/financeReceiptWriteOffNo")
    @ApiOperation("收款单反核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> createFinanceReceiptWriteOffNo(@RequestBody FinanceReceiptVo financeReceiptVo) {
        receiptService.createFinanceReceiptWriteOffNo(financeReceiptVo);
        return success(true);
    }

    /*
      批量 收款单反核销
     */
    @PostMapping("/financeReceiptWriteOffNo2")
    @ApiOperation("收款单反核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> createFinanceReceiptWriteOffNo2(
            @RequestBody List<FinanceReceiptVo> financeReceiptVoList) {
        //集合类通用工具
        if (CollectionUtil.isEmpty(financeReceiptVoList)) {
            return success(true);
        }
        for (FinanceReceiptVo financeReceiptVo :
                financeReceiptVoList) {
            receiptService.createFinanceReceiptWriteOffNo(financeReceiptVo);
        }
        return success(true);
    }

    @PostMapping("/cancelFinanceReceiptWriteOffNo")
    @ApiOperation("取消收款单反核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinanceReceiptWriteOffNo(@RequestBody FinanceReceiptCancelVo createReqVO) {
        receiptService.cancelFinanceReceiptWriteOffNo(createReqVO);
        return success(true);
    }


    @GetMapping("/getReceiptApprove")
    @ApiOperation("根据流程主键获取收款单流程审核信息")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<ReceiptApprovalBackVO> getReceiptApprove(@RequestParam("id") Long id) {
        ReceiptApprovalDO paymentApprovalDO = receiptApprovalService.getReceiptApproval(id);
        return success(ReceiptApprovalConvert.INSTANCE.convert(paymentApprovalDO));
    }


    @PutMapping("/update")
    @ApiOperation("更新收款单")
    //@PreAuthorize("@ss.hasPermission('ecw:receipt:update')")
    public CommonResult<Boolean> updateReceipt(@Valid @RequestBody ReceiptUpdateReqVO updateReqVO) {
        receiptService.updateReceipt(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除收款单")
    @ApiImplicitParam(name = "id", value = "应收款单ID", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:receipt:delete')")
    public CommonResult<Boolean> deleteReceipt(@RequestParam("id") Long id) {
        receiptService.deleteReceipt(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("根据收款单ID获得收款单详细信息")
    @ApiImplicitParam(name = "id", value = "应收款单ID", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:receipt:query')")
    public CommonResult<ReceiptBackVO> getReceiptDetail(@RequestParam("id") Long id) {
        return success(receiptService.getReceiptDetail(id));
    }


    @GetMapping("/getReceivableItemDetail")
    @ApiOperation("根据收款单ID获得应收明细列表")
    @ApiImplicitParam(name = "id", value = "应收款单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<List<ReceivableBackVO>> getReceivableItemDetail(@RequestParam("id") Long id) {
        return success((receivableService.getReceivableListByReceiptId(id)));
    }

    @GetMapping("/getReceiptAccountList")
    @ApiOperation("根据收款单ID获得收款账户细列表")
    @ApiImplicitParam(name = "id", value = "应收款单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<List<ReceiptAccountBackVO>> getReceiptAccountList(@RequestParam("id") Long id) {
        LambdaQueryWrapper<ReceiptAccountDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReceiptAccountDO::getReceiptId, id);
        List<ReceiptAccountDO> list = receiptAccountService.list(lambdaQueryWrapper);
        return success(ReceiptAccountConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/list")
    @ApiOperation("获得收款单列表")
    @ApiImplicitParam(name = "ids", value = "编号列表", required = true, example = "1024,2048", dataTypeClass = List.class)
    //@PreAuthorize("@ss.hasPermission('ecw:receipt:query')")
    public CommonResult<List<ReceiptBackVO>> getReceiptList(@RequestParam("ids") Collection<Long> ids) {
        List<ReceiptDO> list = receiptService.getReceiptList(ids);
        return success(ReceiptConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/getReceiptAccountList/{id}")
    @ApiOperation("根據收款單得到應收賬號")
    @ApiImplicitParam(name = "id", value = "收款单ID", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<List<ReceiptAccountBackVO>> getReceiptAccountItem(@PathVariable("id") Long id) {
        LambdaQueryWrapper<ReceiptAccountDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReceiptAccountDO::getReceiptId, id);
        List<ReceiptAccountDO> accountList = receiptAccountService.list(lambdaQueryWrapper);
        return success(ReceiptAccountConvert.INSTANCE.convertList(accountList));
    }

    @GetMapping("/bank-writeoff-range")
    @ApiOperation("获得银行收款明细自动核销比例")
    public CommonResult<ReceiptBankWriteoffRangeVO> getBankWriteoffRange() {
        return success(receiptService.getBankWriteoffRange());
    }

    @PostMapping("/bank-writeoff-range")
    @ApiOperation("设置银行收款明细自动核销比例")
    public CommonResult<Boolean> getBankWriteoffRange(@RequestBody ReceiptBankWriteoffRangeVO range) {
        return success(receiptService.setBankWriteoffRange(range));
    }

    @PostMapping("/batch-gen-receipt")
    @ApiOperation("批量生成收款单")
    @Idempotent(timeout = 5)
    public CommonResult<List<ReceiptBatchGenRespVO>> batchGenReceipt(@RequestBody List<ReceiptBatchCreateReqVO> createReqVOList) {
        return success(receiptService.batchGenReceipt(createReqVOList));
    }


    @GetMapping("/export-excel")
    @ApiOperation("导出收款单 Excel")
    //@PreAuthorize("@ss.hasPermission('ecw:receipt:export')")
    @OperateLog(type = EXPORT)
    public void exportReceiptExcel(@Valid ReceiptExportVO query, HttpServletResponse response) throws Exception {
        response.setContentType("application/octet-stream");
        response.setHeader("content-type", "application/octet-stream");
        response.setCharacterEncoding("utf-8");
        response.addHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("收款单.xlsx", "UTF-8").replaceAll("\\+", "%20"));

        ReceiptDO receiptDO = receiptService.getReceipt(query.getId());

        if (null == receiptDO) {
            throw ServiceExceptionUtil.exception(PAYMENT_NOT_EXISTS);
        }

        //得到收款单的始发地
        List<ReceivableBackVO> receivableList = receivableService.getReceivableListByReceiptId(receiptDO.getId());

        ReceiptExportDto receiptExportDto = receiptService.getReceiptExportByOrderId(receivableList.get(0).getOrderId());

        ClassPathResource classPathResource = buildClassPathResource(receiptExportDto.getOriginWarehouse(), query.getBillType());
        //得到每个模板的起始位置
        int rowIndex = buildStartIndex(receiptExportDto.getOriginWarehouse(), query.getBillType());


        LambdaQueryWrapper<ReceiptAccountDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReceiptAccountDO::getReceiptId, receiptDO.getId());
        List<ReceiptAccountDO> accountList = receiptAccountService.list(lambdaQueryWrapper);
        Map<Long, ReceiptAccountDO> distanctMap = new HashMap<>();
        for (ReceiptAccountDO receiptAccountDO : accountList) {
            //receiptAccountDO.getCurrencyId();
            //receiptAccountDO.getCollectionRate();
            //receiptAccountDO.getCollectionCurrencyId();
            distanctMap.put(receiptAccountDO.getCurrencyId(), receiptAccountDO);
        }


        Workbook workbook = WorkbookFactory.create(classPathResource.getInputStream());


        Font font = workbook.createFont();
        font.setFontName("宋体");
        font.setFontHeightInPoints((short) 14);
        font.setBold(true);

        CellStyle cellstyle = workbook.createCellStyle();
        cellstyle.setVerticalAlignment(VerticalAlignment.CENTER);
        cellstyle.setAlignment(HorizontalAlignment.CENTER);
        cellstyle.setBorderBottom(BorderStyle.THIN); //下边框
        cellstyle.setBorderLeft(BorderStyle.THIN);//左边框
        cellstyle.setBorderTop(BorderStyle.THIN);//上边框
        cellstyle.setBorderRight(BorderStyle.THIN);//右边框
        cellstyle.setFont(font);
        cellstyle.setWrapText(true);

        Sheet sheet = workbook.getSheetAt(0);

        DictDataRespDTO transportType = null;
        //得到所有币种--后面使用
        Map<Integer, CurrencyRespDTO> currencyDoMap = currencyApi.getAllCurrency();
        log.error(currencyDoMap.toString());

        //收款项
        //List<ReceivableBackVO> receivableList = receivableService.getReceivableListByReceiptId(receiptDO.getId());

        //Map<Long, BigDecimal> distanctMap = new HashMap<>();
        short rowHeight = sheet.getRow(rowIndex).getHeight();

        if (null != receivableList && receivableList.size() > 0) {


            //重设图片防止被拉伸
            XSSFDrawing drawings = (XSSFDrawing) sheet.getDrawingPatriarch();
            if (null != drawings && drawings.getShapes().size() >= 2) {
                XSSFShape shape = drawings.getShapes().get(0);
                if (shape instanceof Picture) {

                    XSSFPicture picture = (XSSFPicture) shape;
                    XSSFClientAnchor anchor = picture.getClientAnchor();
                    int row1 = anchor.getRow1();
                    int row2 = anchor.getRow2();
                    anchor.setRow1(row1 + receivableList.size());
                    anchor.setRow2(row2 + receivableList.size());
                }
            }

            //插入指定的行数
            sheet.shiftRows(rowIndex, sheet.getLastRowNum(), receivableList.size(), true, false);


            for (int i = 0; i < receivableList.size(); i++) {
                ReceivableBackVO receivableBackVO = receivableList.get(i);
                DictDataRespDTO data = DictFrameworkUtils.getDictDataFromCache("receivable_fee_type", String.valueOf(receivableBackVO.getFeeType()));
                if (null == transportType) {
                    transportType = DictFrameworkUtils.getDictDataFromCache("transport_type", String.valueOf(receivableBackVO.getTransportId()));
                }
                receivableBackVO.setFeeTypeLabel(data.getLabel());
                receivableBackVO.setFeeTypeLabelEn(data.getLabelEn());

                CurrencyRespDTO currencyRespDTO = currencyDoMap.get(receivableBackVO.getCurrencyId().intValue());

                Row row = sheet.createRow(rowIndex + i);
                row.setHeight(rowHeight);

                Cell targetCell = null;

                //提货单号
                targetCell = row.createCell(0);
                targetCell.setCellStyle(cellstyle);
                targetCell.setCellValue(receivableBackVO.getTidanNo());

                //订单号
                targetCell = row.createCell(1);
                targetCell.setCellStyle(cellstyle);
                targetCell.setCellValue(receivableBackVO.getOrderNo());

                //箱量
                targetCell = row.createCell(2);
                targetCell.setCellStyle(cellstyle);
                targetCell.setCellValue(receivableBackVO.getNum());

                //品名
                targetCell = row.createCell(3);
                targetCell.setCellStyle(cellstyle);
                targetCell.setCellValue(StrUtil.nullToDefault(receivableBackVO.getTitleZh(), "") + "/" + StrUtil.nullToDefault(receivableBackVO.getTitleEn(), ""));

                //费用类型
                targetCell = row.createCell(4);
                targetCell.setCellStyle(cellstyle);
                targetCell.setCellValue(StrUtil.nullToDefault(receivableBackVO.getFeeTypeLabel(), "") + "/" + StrUtil.nullToDefault(receivableBackVO.getFeeTypeLabelEn(), ""));


                //体积
                targetCell = row.createCell(5);
                targetCell.setCellStyle(cellstyle);
                targetCell.setCellValue(receivableBackVO.getVolume().doubleValue());

                //重量
                targetCell = row.createCell(6);
                targetCell.setCellStyle(cellstyle);
                targetCell.setCellValue(receivableBackVO.getWeight().doubleValue());

                //单价
                targetCell = row.createCell(7);
                targetCell.setCellStyle(cellstyle);
                targetCell.setCellValue(currencyRespDTO.getFuhao() + ":" + receivableBackVO.getUnitPrice().doubleValue());

                //税率
                targetCell = row.createCell(8);
                targetCell.setCellStyle(cellstyle);
                targetCell.setCellValue(receivableBackVO.getTaxPoint().doubleValue());

                log.error(currencyRespDTO.toString());
                if (receivableBackVO.getTotalAmount() != null) {
                    //总金额
                    targetCell = row.createCell(9);
                    targetCell.setCellStyle(cellstyle);
                    targetCell.setCellValue(currencyRespDTO.getFuhao() + ":" + receivableBackVO.getTotalAmount().doubleValue());
                }

                if (receivableBackVO.getDiscountTotal() != null) {
                    //优惠金额
                    targetCell = row.createCell(10);
                    targetCell.setCellStyle(cellstyle);
                    targetCell.setCellValue(currencyRespDTO.getFuhao() + ":" + receivableBackVO.getDiscountTotal().doubleValue());
                }

                ReceiptAccountDO accountDO = distanctMap.get(receivableBackVO.getCurrencyId());
                if (null != accountDO) {
                    //收款币种
                    if (null != accountDO.getCollectionCurrencyId()) {
                        CurrencyRespDTO respDTO = currencyDoMap.get(accountDO.getCollectionCurrencyId().intValue());
                        if (respDTO != null) {
                            targetCell = row.createCell(11);
                            targetCell.setCellStyle(cellstyle);
                            targetCell.setCellValue(respDTO.getTitleZh() + "(" + respDTO.getTitleEn() + ")");
                        }
                    }
                    //汇率
                    if (null != accountDO.getCollectionRate()) {
                        targetCell = row.createCell(12);
                        targetCell.setCellStyle(cellstyle);
                        targetCell.setCellValue(accountDO.getCollectionRate().doubleValue());
                    }

                    //金额
                    if (receivableBackVO.getTotalAmount() != null && null != accountDO.getCollectionRate()) {
                        //总金额
                        targetCell = row.createCell(13);
                        targetCell.setCellStyle(cellstyle);
                        targetCell.setCellValue(NumberUtil.decimalFormat("#0.00", receivableBackVO.getTotalAmount().multiply(accountDO.getCollectionRate())));
                    }
                }


            }
        }


        Cell cell = null;

        //客户
        cell = sheet.getRow(3).getCell(3);
        cell.setCellValue(receiptDO.getCustomerName());

        //出货方式
        cell = sheet.getRow(3).getCell(9);

        if (StringUtils.equals(query.getBillType(), "2")) {
            cell.setCellValue(transportType.getLabelEn());
        } else {
            cell.setCellValue(transportType.getLabel());
        }


        //账单有效期
        cell = sheet.getRow(6).getCell(3);
        cell.setCellValue(DateUtils.formatDate(receiptDO.getRateValidateDate()));

        //制单日期
        cell = sheet.getRow(6).getCell(9);
        cell.setCellValue(DateUtils.formatDate(receiptDO.getCreateTime()));

        //目的国
        cell = sheet.getRow(4).getCell(9);
        if (StringUtils.equals(query.getBillType(), "2")) {
            cell.setCellValue(receiptExportDto.getDestCountryEn());
        } else {
            cell.setCellValue(receiptExportDto.getDestCountryZh());
        }

        //目的港
        cell = sheet.getRow(5).getCell(9);
        cell.setCellValue(StrUtil.emptyToDefault(receiptExportDto.getDestPortZh(), "") + "/" + StrUtil.emptyToDefault(receiptExportDto.getDestPortEn(), ""));

        if (StringUtils.equals("1", query.getBillType())) {
            //转成美金--需要得得当前币种和美金的汇率
            rowIndex = rowIndex + receivableList.size();
            cell = sheet.getRow(rowIndex).getCell(10);


            BigDecimal rmbTotal = BigDecimal.ZERO;
            BigDecimal usdTotal = BigDecimal.ZERO;

            if (null != receivableList && receivableList.size() > 0) {
                for (ReceivableBackVO receivableBackVO : receivableList) {
                    CurrencyRespDTO currencyRespDTO = currencyDoMap.get(receivableBackVO.getCurrencyId().intValue());
                    if (StrUtil.equals(currencyRespDTO.getFuhao(), "￥")) {
                        rmbTotal = rmbTotal.add(receivableBackVO.getTotalAmount());
                    } else {
                        if (StrUtil.isNotBlank(currencyRespDTO.getFuhao())) {
                            ExchangeRateRespDTO exchangeRateRespDTO = currencyApi.getCurrencyRateByCode(currencyRespDTO.getFuhao(), "$");
                            usdTotal = usdTotal.add(receivableBackVO.getTotalAmount().multiply(exchangeRateRespDTO.getCurrencyRate()));
                        }
                    }

                }
            }

            cell.setCellValue(NumberUtil.decimalFormat("#0.00", usdTotal));

            cell = sheet.getRow(rowIndex).getCell(12);
            cell.setCellValue(NumberUtil.decimalFormat("#0.00", rmbTotal));
        }

        if (StringUtils.equals("2", query.getBillType())) {


            //转成美金--需要得得当前币种和美金的汇率
            rowIndex = rowIndex + receivableList.size();
            cell = sheet.getRow(rowIndex).getCell(10);


            BigDecimal rmbTotal = BigDecimal.ZERO;
            BigDecimal usdTotal = BigDecimal.ZERO;

            if (null != receivableList && receivableList.size() > 0) {
                for (ReceivableBackVO receivableBackVO : receivableList) {
                    CurrencyRespDTO currencyRespDTO = currencyDoMap.get(receivableBackVO.getCurrencyId().intValue());
                    if (StrUtil.equals(currencyRespDTO.getFuhao(), getCurrencyFuhao(receiptExportDto.getDestCountryZh()))) {
                        rmbTotal = rmbTotal.add(receivableBackVO.getTotalAmount());
                    } else {
                        if (StrUtil.isNotBlank(currencyRespDTO.getFuhao())) {
                            ExchangeRateRespDTO exchangeRateRespDTO = currencyApi.getCurrencyRateByCode(currencyRespDTO.getFuhao(), "$");

                            usdTotal = usdTotal.add(receivableBackVO.getTotalAmount().multiply(exchangeRateRespDTO.getCurrencyRate()));

                        }
                    }

                }
            }

            cell.setCellValue(NumberUtil.decimalFormat("#0.00", usdTotal));

            cell = sheet.getRow(rowIndex).getCell(11);
            cell.setCellValue(getCurrencyTitileEn(receiptExportDto.getDestCountryZh()));

            cell = sheet.getRow(rowIndex).getCell(12);
            cell.setCellValue(NumberUtil.decimalFormat("#0.00", rmbTotal));

        }


        /*CurrencyRespDTO dto = currencyDoMap.get(receiptDO.getCurrencyId().intValue());
        ExchangeRateRespDTO exchangeRateRespDTO = currencyApi.getCurrencyRateByCode(dto.getFuhao(), "$");
        cell.setCellValue(NumberUtil.decimalFormat("#0.00", receiptDO.getReceivableTotalAmount().multiply(exchangeRateRespDTO.getCurrencyRate())));

        if (StringUtils.equals("1", query.getBillType())) {
            //转成RMB--需要得得当前币种和RMB的汇率
            cell = sheet.getRow(rowIndex).getCell(12);
            ExchangeRateRespDTO exchangeRateRespDTO1 = currencyApi.getCurrencyRateByCode(dto.getFuhao(), "￥");
            cell.setCellValue(NumberUtil.decimalFormat("#0.00", receiptDO.getReceivableTotalAmount().multiply(exchangeRateRespDTO1.getCurrencyRate())));
        }

        if (StringUtils.equals("2", query.getBillType())) {
            //国外账单时要根据目的国转成国外币种
            cell = sheet.getRow(rowIndex).getCell(11);
            cell.setCellValue(getCurrencyTitileEn(receiptExportDto.getDestCountryZh()));


            cell = sheet.getRow(rowIndex).getCell(12);
            ExchangeRateRespDTO exchangeRateRespDTO1 = currencyApi.getCurrencyRateByCode(dto.getFuhao(), getCurrencyFuhao(receiptExportDto.getDestCountryZh()));
            cell.setCellValue(NumberUtil.decimalFormat("#0.00", receiptDO.getReceivableTotalAmount().multiply(exchangeRateRespDTO1.getCurrencyRate())));
        }*/
        ServletOutputStream outputStream = response.getOutputStream();
        workbook.write(outputStream);
        IoUtil.close(outputStream);
    }

    private ClassPathResource buildClassPathResource(String originWarehouse, String type) {
        //加载国外模板
        if (StringUtils.equals(type, "1")) {
            if (originWarehouse.contains("广州")) {
                return new ClassPathResource("templates/SKD00000003.xlsx");
            } else if (originWarehouse.contains("义乌")) {
                return new ClassPathResource("templates/SKD00000005.xlsx");
            } else {
                return new ClassPathResource("templates/SKD00000003.xlsx");
            }
        } else if (StringUtils.equals(type, "2")) {
            return new ClassPathResource("templates/SKD00000004.xlsx");
        } else {
            return new ClassPathResource("templates/SKD00000004.xlsx");
        }
    }

    private int buildStartIndex(String originWarehouse, String type) {
        //加载国外模板
        if (StringUtils.equals(type, "1")) {
            if (originWarehouse.contains("广州")) {
                return 8;
            } else if (originWarehouse.contains("义乌")) {
                return 8;
            } else {
                return 8;
            }
        } else if (StringUtils.equals(type, "2")) {
            return 8;
        } else {
            return 8;
        }
    }

    public String getCurrencyTitileEn(String destCountry) {
        if (destCountry.contains("科特")) {
            return "XOF：";
        } else if (destCountry.contains("尼日利亚")) {
            return "NGN:";
        } else if (destCountry.contains("加纳")) {
            return "GHS:";
        } else {
            return "RMB:";
        }
    }

    public String getCurrencyFuhao(String destCountry) {
        if (destCountry.contains("科特")) {
            return "xof";
        } else if (destCountry.contains("尼日利亚")) {
            return "N";
        } else if (destCountry.contains("加纳")) {
            return "GH₵";
        } else {
            return "￥";
        }
    }

    @GetMapping("/updateReceiptNewFeild")
    @ApiOperation("更新应收明细")
    public CommonResult<String> updateReceiptNewField() throws Exception {
        return success(updateReceiptNewFieldTask.execute(""));
    }
}
